"use strict";
this.author			= "phkb"; 
this.copyright		= "© 2017 phkb"; 
this.licence		= "CC-BY-NC-SA 4.0";
this.name			= "Waypoint_Here_MFD";
this.description	= "Waypoint Here MFD Script";

//-------------------------------------------------------------------------------------------------------------
this.startUpComplete = function() {
	// add interface to HUD selector
	var h = worldScripts.hudselector;
	if (h) h.$HUDSelectorAddMFD(this.name, this.name);
	h = null;
}

//-------------------------------------------------------------------------------------------------------------
this.shipWillLaunchFromStation = function(station) {
    // only start the waypoint timer if we have the equipment installed
    if (player.ship.equipmentStatus("EQ_WPH_ASC_UPGRADE") === "EQUIPMENT_OK")
        this._updateWaypointMFD = new Timer(this, this.$updateWaypointMFD, 5, 0.5);
}

//-------------------------------------------------------------------------------------------------------------
this.shipDied = function(whom, why) {
	this.$stopTimer();
}

//-------------------------------------------------------------------------------------------------------------
this.shipWillDockWithStation = function(station) {
    this.$stopTimer();
}

//-------------------------------------------------------------------------------------------------------------
this.$stopTimer = function() {
    if (this._updateWaypointMFD && this._updateWaypointMFD.isRunning) this._updateWaypointMFD.stop();
}

//-------------------------------------------------------------------------------------------------------------
// updates the text of the damage report MFD
this.$updateWaypointMFD = function $updateWaypointMFD() {
    var p = player.ship;
    var output = "SYSTEM WAYPOINTS:\n";
    var keys = [];
    if (system.waypoints) keys = Object.keys(system.waypoints);
    if (keys.length === 0) {
        output += "\n  No waypoints set";
    } else {
        for (var i = 0; i < keys.length; i++) {
            if (i > 8) break;
			var wp = system.waypoints[keys[i]];
            output += this.$padTextRight(
				(p.compassTarget && p.compassTarget == wp ? "•" : "") + 
				(wp.beaconLabel !== "" ? wp.beaconLabel : "(Unlabelled)"), 
				(system.isInterstellarSpace === true ? 7.5 : 8.5)
				) +	(wp.position.distanceTo(p) / 1000).toFixed(1) + "kms";
			// give the player some waypoint direction information in interstellar space
			// because the space compass doesn't work
			if (system.isInterstellarSpace === true && p && p.vectorForward) {
				var f_dev = p.vectorForward.angleTo(wp.position.subtract(p));
				var r_dev = p.vectorRight.angleTo(wp.position.subtract(p));
				var u_dev = p.vectorUp.angleTo(wp.position.subtract(p));
				var s = "";
				// > 1.56 means oppposite side (3.12 exact opposite)
				// so, f_dev < 0.01, r_dev = 1.57, u_dev = 1.57  -- aligned
				if (f_dev < 0.25) {
					if (f_dev < 0.25) s = "O";
					if (f_dev < 0.12) s = "o";
					if (f_dev < 0.05) s = "X";
				}
				if (f_dev >= 0.25) {
					if (r_dev > 1.69) {
						s += "<";
					}
					if (u_dev < 1.45) {
						s += "^";
					} else if(u_dev > 1.69) {
						s += "v";
					}
					if (r_dev < 1.45) {
						s += ">";
					}
					if (f_dev > 1.57) s+= "A";
				}
				output += "  " + s;
			}
			output += "\n";
        }
    }
	// set the text in the MFD
	p.setMultiFunctionText(this.name, output, false);
}

//-------------------------------------------------------------------------------------------------------------
// appends space to currentText to the specified length in 'em'
this.$padTextRight = function(currentText, desiredLength, leftSwitch) {
	if (currentText == null) currentText = "";
	var hairSpace = String.fromCharCode(31);
	var ellip = "…";
	var currentLength = defaultFont.measureString(currentText.toString().replace(/%%/g, "%"));
	var hairSpaceLength = defaultFont.measureString(hairSpace);
	// calculate number needed to fill remaining length
	var padsNeeded = Math.floor((desiredLength - currentLength) / hairSpaceLength);
	if (padsNeeded < 1)	{
		// text is too long for column, so start pulling characters off
		var tmp = currentText;
		do {
			tmp = tmp.substring(0, tmp.length - 2) + ellip;
			if (tmp == ellip) break;
		} while (defaultFont.measureString(tmp.replace(/%%/g, "%")) > desiredLength);
		currentLength = defaultFont.measureString(tmp);
		padsNeeded = Math.floor((desiredLength - currentLength) / hairSpaceLength);
		currentText = tmp;
	}
	// quick way of generating a repeated string of that number
	if (!leftSwitch || leftSwitch == false) {
		return currentText + new Array(padsNeeded).join(hairSpace);
	} else {
		return new Array(padsNeeded).join(hairSpace) + currentText;
	}
}
